package com.sg.answer.service;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.sg.answer.core.HttpRequestConfig;
import com.sg.answer.core.HttpRequestResult;
import com.sg.answer.core.SgEntity;
import com.sg.answer.sg.ApiEndpoints;
import com.sg.answer.sg.ErrorCode;
import com.sg.answer.sg.SgException;
import com.sg.answer.utils.HttpClientUtils;
import com.sg.answer.utils.SgUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.CookieStore;
import org.apache.http.client.HttpClient;
import org.apache.http.protocol.HttpContext;

import java.io.IOException;
import java.util.List;
import java.util.Random;

import static com.sg.answer.sg.Constant.answerNum;

@Slf4j
public class AnswerService {

    private final CookieStore cookieStore;

    public AnswerService(CookieStore cookieStore) {
        this.cookieStore = cookieStore;
    }

    private void openAnswerHtml(int num) throws Exception {
        String openAnswerUrl = ApiEndpoints.START_LEVEL + "?id=" + num + "&matchid=0";
        log.info("闯关页面请求地址:{}", openAnswerUrl);
        String openResult = SgUtils.executeRemotePostRequest(cookieStore, openAnswerUrl);
        if (StringUtils.isEmpty(openResult)) {
            log.error("请求返回结果集为空");
            throw new SgException(ErrorCode.RESULT_EMPTY.getCode(), ErrorCode.RESULT_EMPTY.getName());
        }
        if (openResult.contains("今天的闯关时间已用尽")) {
            log.error("今天的闯关时间已用尽");
            throw new SgException(ErrorCode.TIME_Limit.getCode(), ErrorCode.TIME_Limit.getName());
        }
        if (openResult.contains("未解锁")) {
            log.error("当前关卡未解锁");
            throw new SgException(ErrorCode.LEVEL_NOT_UNLOCKED.getCode(), ErrorCode.LEVEL_NOT_UNLOCKED.getName());
        }
    }

    public JSONObject autoAnswer(List<String> stemList, String num) throws Exception {
        String answerResult;
        int index = new Random().nextInt(50);
        String stem = stemList.get(index);
        log.info("随机选题：序号={}，题干={}", index, stem);
        String dtUrl = ApiEndpoints.SUBMIT_ANSWER + "?passid=" + num + "&" + stem;
        try {
            answerResult = SgUtils.executeRemotePostRequest(cookieStore, dtUrl);
        } catch (Exception e) {
            log.error("请求超时，10秒后重试...");
            Thread.sleep(10000);
            answerResult = SgUtils.executeRemotePostRequest(cookieStore, dtUrl);
        }
        return handleAnswerResult(answerResult, dtUrl);
    }

    public JSONObject handleAnswerResult(String res,String dtUrl) throws Exception {
        JSONObject jb;
        try {
            jb = JSONObject.parseObject(res);
//          log.info("当前答题数量:{}",jb.get("isright"));
        } catch (JSONException exception) {
            log.error("json解析异常{}", exception.getMessage());
            if (exception.getMessage().contains("重新登录")) {
                throw new SgException(ErrorCode.LOGOUT.getCode(), ErrorCode.LOGOUT.getName());
            }
            Thread.sleep(10000);
            String tryResult = SgUtils.executeRemotePostRequest(cookieStore, dtUrl);
            jb = JSONObject.parseObject(tryResult);
        }
        return jb;
    }
}
